/*
 * This file is part of the Generic Data Structures Library (GDSL).
 * Copyright (C) 1998-2004 Nicolas Darnis <ndarnis@free.fr>.
 *
 * The GDSL library is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The GDSL library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the GDSL library; see the file COPYING.
 * If not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 *
 * $RCSfile: gdsl_types.h,v $
 * $Revision: 1.21 $
 * $Date: 2004/09/30 11:00:48 $
 */

#ifndef _GDSL_TYPES_H_
#define _GDSL_TYPES_H_


#include <stdio.h>


#ifdef __cplusplus
extern "C" 
{
#endif /* __cplusplus */


/**
 * @defgroup gdsl_types GDSL types
 * @{
 */


/**
 * @brief GDSL Constants.
 */
typedef enum
{
    /** Memory allocation error */
    GDSL_ERR_MEM_ALLOC = -1,

    /** For stopping a parsing function */
    GDSL_MAP_STOP = 0,

    /** For continuing a parsing function */
    GDSL_MAP_CONT = 1,

    /** To indicate an inserted value */
    GDSL_INSERTED,

    /** To indicate a founded value */
    GDSL_FOUND

} gdsl_constant_t;

/**
 * @brief GDSL element type.
 *
 * All GDSL internal data structures contains a field of this type. This field 
 * is for GDSL users to store their data into GDSL data structures.
 */
typedef void* gdsl_element_t;

/**
 * @brief GDSL Alloc element function type.
 *
 * This function type is for allocating a new gdsl_element_t variable.
 * The USER_DATA argument should be used to fill-in the new element.
 *
 * @param USER_DATA user data used to create the new element
 * @return the newly allocated element in case of success.
 * @return NULL in case of failure.
 * @see gdsl_free_func_t
 */
typedef gdsl_element_t 
(* gdsl_alloc_func_t) (void* USER_DATA
		       );

/**
 * @brief GDSL Free element function type.
 *
 * This function type is for freeing a gdsl_element_t variable.
 * The element must have been previously allocated by a function of 
 * gdsl_alloc_func_t type.
 * A free function according to gdsl_free_func_t must free the ressources 
 * allocated by the corresponding call to the function of type 
 * gdsl_alloc_func_t. The GDSL functions doesn't check if E != NULL before 
 * calling this function.
 *
 * @param E The element to deallocate
 * @see gdsl_alloc_func_t
 */
typedef void 
(* gdsl_free_func_t) (gdsl_element_t E
		      );

/**
 * @brief GDSL Copy element function type.
 *
 * This function type is for copying gdsl_element_t variables.
 *
 * @param E The gdsl_element_t variable to copy
 * @return the copied element in case of success.
 * @return NULL in case of failure.
 */
typedef gdsl_element_t 
(* gdsl_copy_func_t) (const gdsl_element_t E
		      );

/**
 * @brief GDSL Map element function type.
 *
 * This function type is for mapping a gdsl_element_t variable from a GDSL
 * data structure.
 * The optional USER_DATA could be used to do special thing if needed.
 *
 * @param E The actually mapped gdsl_element_t variable
 * @param USER_DATA User's datas
 * @return GDSL_MAP_STOP if the parsing must be stopped.
 * @return GDSL_MAP_CONT if the parsing must continue.
 */
typedef int 
(* gdsl_map_func_t) (const gdsl_element_t E,
		     void* USER_DATA
		     );

/**
 * @brief GDSL Comparison element function type.
 *
 * This function type is used to compare a gdsl_element_t variable with a user
 * value.
 * The E argument is always the one in the GDSL data structure, VALUE is always 
 * the one the user wants to compare E with.
 *
 * @param E The gdsl_element_t variable contained into the data structure to
 * compare from
 * @param VALUE The user data to compare E with
 * @return < 0  if E is assumed to be less than VALUE.
 * @return 0 if E is assumed to be equal to VALUE.
 * @return > 0 if E is assumed to be greather than VALUE.
 */
typedef int 
(* gdsl_compare_func_t) (const gdsl_element_t E,
			 void* VALUE
			 );

/**
 * @brief GDSL Write element function type.
 *
 * This function type is for writing a gdsl_element_t E to OUTPUT_FILE. 
 * Additional USER_DATA could be passed to it.
 *
 * @param E The gdsl_element_t variable to write
 * @param OUTPUT_FILE The FILE where to write E
 * @param USER_DATA User's datas
 */
typedef void 
(* gdsl_write_func_t) (const gdsl_element_t E,
		       FILE* OUTPUT_FILE,
		       void* USER_DATA
		       );
  
#ifdef _NO_LIBGDSL_TYPES_

#else /* _NO_LIBGDSL_TYPES_ */

#include <sys/types.h>

#ifdef HAVE_LINUX

typedef char          byte;
typedef unsigned char ubyte;

#else /* not HAVE_LINUX */

#ifndef _SYS_TYPES_H
typedef unsigned long int ulong;
#endif /* _SYS_TYPES_H */

#endif /* not HAVE_LINUX */

#ifndef __cplusplus

#ifdef TRUE
#undef TRUE
#endif

#ifdef FALSE
#undef FALSE
#endif

#ifdef bool
#undef bool
#endif /* not __cplusplus */

/**
 * GDSL boolean type.
 * Defines _NO_LIBGDSL_TYPES_ at compilation time if you don't want them.
 */
typedef enum 
{
  /** FALSE boolean value */
  FALSE = 0,
  /** TRUE boolean value */
  TRUE = 1 
} bool;

#endif /* not __cplusplus */

#endif /* not _NO_LIBGDSL_TYPES_ */


/*
 * @}
 */


#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* _GDSL_TYPES_H_ */

/** EMACS **
 * Local variables:
 * mode: c
 * c-basic-offset: 4
 * End:
 */
